# Common rules for building a Minibloq application
# on an arduino platform


# Define programs and commands.
ifeq ($(OS),Windows_NT)
  TOOLS=$(TOOLSPATH)/WinAvr/v20090313/bin
  REMOVE = $(TOOLSPATH)/WinAvr/v20090313/utils/bin/rm -f
else
  ifeq ($(shell getconf LONG_BIT),64)
    ARCH=amd64
  else
    ARCH=i386
  endif
  TOOLS=$(TOOLSPATH)/avrlinux/$(ARCH)/v1.0/bin
  REMOVE = rm -f
endif

CC = $(TOOLS)/avr-gcc
OBJCOPY = $(TOOLS)/avr-objcopy
OBJDUMP = $(TOOLS)/avr-objdump
SIZE = $(TOOLS)/avr-size
AR = $(TOOLS)/avr-ar rcs
NM = $(TOOLS)/avr-nm
AVRDUDE = $(TOOLS)/avrdude


.PHONY: build program clean 

build: $(OUTPUTPATH)/$(TARGET).hex $(OUTPUTPATH)/$(TARGET).eep size

# NOTE: it's important to remove the .pde file otherwise
#       Minibloq won't build!!!!
clean:
	$(REMOVE) $(BLOCK_OBJ)
	$(REMOVE) $(OUTPUTPATH)/$(TARGET).o
	$(REMOVE) $(OUTPUTPATH)/$(TARGET).elf
	$(REMOVE) $(OUTPUTPATH)/$(TARGET).hex
	$(REMOVE) $(OUTPUTPATH)/$(TARGET).eep
	$(REMOVE) $(OUTPUTPATH)/$(TARGET).pde

program: build
	$(AVRDUDE) $(AVRDUDE_FLAGS) $(AVRDUDE_WRITE_FLASH) $(AVRDUDE_WRITE_EEPROM)



$(OUTPUTPATH)/$(TARGET).elf: $(BLOCK_OBJ) $(OUTPUTPATH)/$(TARGET).o
	$(CC) $(LDFLAGS) -o $@ $(OUTPUTPATH)/$(TARGET).o $(BLOCK_OBJ) $(COREPATH)/core.a
	
$(OUTPUTPATH)/$(TARGET).hex: $(OUTPUTPATH)/$(TARGET).elf
	$(OBJCOPY) -O ihex -R .eeprom $< $@

$(OUTPUTPATH)/$(TARGET).eep: $(OUTPUTPATH)/$(TARGET).elf
	$(OBJCOPY) -O ihex -j .eeprom --set-section-flags=.eeprom=alloc,load --no-change-warnings --change-section-lma .eeprom=0 $< $@
	
size: $(OUTPUTPATH)/$(TARGET).elf
	$(SIZE) --mcu=$(MCU) --format=avr $<
	
$(OUTPUTPATH)/$(TARGET).o: $(MAIN_FILE) $(OUTPUTPATH)/$(TARGET).pde
	$(CC) $(CFLAGS) $(INCLUDE_DIRS) $(BLOCK_INCLUDES) $(MAIN_FILE) -o $@

#Create targets for all .cpp files	
#TODO: handle .c files
define BLOCK_template
$(addprefix $(OUTPUTPATH)/,$(patsubst %.cpp,%.o,$(1))): $(BLOCKSPATH)/$(2)/Sources/$(1)
	$(CC) $(CFLAGS) $(INCLUDE_DIRS) $$< -o $$@
endef

$(foreach prog,$(BLOCKS),$(foreach src,$($(prog)_SRC),$(eval $(call BLOCK_template,$(src),$($(prog)_DIR)))))     
